
use strict;
use warnings;

package Zest::Author::TABULO::Util;
our $VERSION = '1.000011';


#region: #== UTILITY FUNCTIONS (EXPORT_OK) ==
#endregion (UTILITY FUNCTIONS)

1;

=pod

=encoding UTF-8

=for :stopwords Tabulo[n]

=head1 NAME

Zest::Author::TABULO::Util - Utility functions used by TABULO's authoring dist

=head1 VERSION

version 1.000011

=head1 AUTHORS

Tabulo[n] <dev@tabulo.net>

=head1 LEGAL

This software is copyright (c) 2022 by Tabulo[n].

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut

__END__

#ABSTRACT: Utility functions used by TABULO's authoring dist





